@@ -44,6 +44,9 @@
 #include "XrdSys/XrdSysLogger.hh"
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSut/XrdSutRndm.hh"
+
+#include "XrdTls/XrdTlsContext.hh"
+
 #include "XrdCrypto/XrdCryptosslTrace.hh"
 
 #include "XrdVersion.hh"
@@ -56,76 +59,25 @@
 static XrdSysLogger Logger;
 static XrdSysError eDest(0,"cryptossl_");
 
-// Mutexes for OpenSSL
-XrdSysMutex *XrdCryptosslFactory::CryptoMutexPool[SSLFACTORY_MAX_CRYPTO_MUTEX];
-
-/******************************************************************************/
-/*             T h r e a d - S a f e n e s s   F u n c t i o n s              */
-/******************************************************************************/
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-#ifdef __solaris__
-extern "C" {
-#endif
-static unsigned long sslfactory_id_callback(void) {
-  return (unsigned long)XrdSysThread::ID();
-}
-
-static void sslfactory_lock(int mode, int n, const char *file, int line)
-{
-  if (mode & CRYPTO_LOCK) {
-    if (XrdCryptosslFactory::CryptoMutexPool[n]) {
-      XrdCryptosslFactory::CryptoMutexPool[n]->Lock();
-    }
-  } else {
-    if (XrdCryptosslFactory::CryptoMutexPool[n]) {
-      XrdCryptosslFactory::CryptoMutexPool[n]->UnLock();
-    }
-  }
-}
-#ifdef __solaris__
-}
-#endif
-#endif
-
-
 //______________________________________________________________________________
 XrdCryptosslFactory::XrdCryptosslFactory() :
                      XrdCryptoFactory("ssl",XrdCryptosslFactoryID)
 {
    // Constructor: init the needed components of the OpenSSL library
-   EPNAME("sslFactory::XrdCryptosslFactory");
 
    // Init SSL ...
-   SSL_library_init();
-   //  ... and its error strings
-   SSL_load_error_strings();
-   // Load Ciphers
-   OpenSSL_add_all_ciphers();
-   // Load Msg Digests
-   OpenSSL_add_all_digests();
+   //
+   static const char *eText = XrdTlsContext::Init();
 
-   if (SSLFACTORY_MAX_CRYPTO_MUTEX < CRYPTO_num_locks() ) {
-      SetTrace(0);
-      TRACE(ALL, "WARNING: do not have enough crypto mutexes as required by crypto_ssl");
-      TRACE(ALL, "        (suggestion: recompile increasing SSLFACTORY_MAX_CRYPTO_MUTEX to "<< CRYPTO_num_locks()<<")");
-   } else {
-      // This code taken from XrdSecProtocolssl thread-safety
-      for (int i = 0; i < SSLFACTORY_MAX_CRYPTO_MUTEX; i++) {
-         XrdCryptosslFactory::CryptoMutexPool[i] = new XrdSysMutex();
+   // Make sure all went well (we need to possibly abort here)
+   //
+   if (eText)
+      {std::cerr <<"XrdCryptosslFactory: " <<eText <<std::endl;
+       abort();
       }
-   }
-
-#if defined(OPENSSL_THREADS)
-   // Thread support enabled: set callback functions
-   CRYPTO_set_locking_callback(sslfactory_lock);
-   CRYPTO_set_id_callback(sslfactory_id_callback);
-#else
-   SetTrace(0);
-   TRACE(ALL, "WARNING: OpenSSL lacks thread support: possible thread-safeness problems!");
-   TRACE(ALL, "         (suggestion: recompile enabling thread support)");
-#endif
 
    // Init Random machinery
+   //
    int klen = 32;
    char *ktmp = XrdSutRndm::GetBuffer(klen);
    if (ktmp) {
@@ -184,6 +136,17 @@ bool XrdCryptosslFactory::SupportedCipher(const char *t)
    return XrdCryptosslCipher::IsSupported(t);
 }
 
+//______________________________________________________________________________
+bool XrdCryptosslFactory::HasPaddingSupport()
+{
+   // Returns true if cipher padding is supported
+#if defined(HAVE_DH_PADDED) || defined(HAVE_DH_PADDED_FUNC)
+   return true;
+#else
+   return false;
+#endif
+}
+
 //______________________________________________________________________________
 XrdCryptoCipher *XrdCryptosslFactory::Cipher(const char *t, int l)
 {
@@ -231,13 +194,29 @@ XrdCryptoCipher *XrdCryptosslFactory::Cipher(XrdSutBucket *b)
    return (XrdCryptoCipher *)0;
 }
 
+//______________________________________________________________________________
+XrdCryptoCipher *XrdCryptosslFactory::Cipher(bool padded, int b, char *p,
+                                             int l, const char *t)
+{
+   // Return an instance of a Ssl implementation of XrdCryptoCipher.
+
+   XrdCryptoCipher *cip = new XrdCryptosslCipher(padded, b,p,l,t);
+   if (cip) {
+      if (cip->IsValid())
+         return cip;
+      else
+         delete cip;
+   }
+   return (XrdCryptoCipher *)0;
+}
+
 //______________________________________________________________________________
 XrdCryptoCipher *XrdCryptosslFactory::Cipher(int b, char *p,
                                              int l, const char *t)
 {
    // Return an instance of a Ssl implementation of XrdCryptoCipher.
 
-   XrdCryptoCipher *cip = new XrdCryptosslCipher(b,p,l,t);
+   XrdCryptoCipher *cip = new XrdCryptosslCipher(false,b,p,l,t);
    if (cip) {
       if (cip->IsValid())
          return cip;
@@ -450,6 +429,15 @@ XrdCryptoX509ParseFile_t XrdCryptosslFactory::X509ParseFile()
    return &XrdCryptosslX509ParseFile;
 }
 
+//______________________________________________________________________________
+XrdCryptoX509ParseStack_t XrdCryptosslFactory::X509ParseStack()
+{
+   // Return an instance of an implementation of a function
+   // to parse a file supposed to contain for X509 certificates.
+
+   return &XrdCryptosslX509ParseStack;
+}
+
 //______________________________________________________________________________
 XrdCryptoX509ParseBucket_t XrdCryptosslFactory::X509ParseBucket()
 {
