@@ -28,12 +28,12 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
   
-#include <ctype.h>
+#include <cctype>
 #include <dirent.h>
 #include <fcntl.h>
-#include <stdlib.h>
+#include <cstdlib>
 #include <strings.h>
-#include <stdio.h>
+#include <cstdio>
 #include <unistd.h>
 #include <sys/param.h>
 #include <sys/stat.h>
@@ -412,7 +412,7 @@ bool XrdDigConfig::ConfigProc(const char *ConfigFN)
             if (!ConfigXeq(var+4, cFile)) {cFile.Echo(); NoGo = 1;}
         }
 
-// Now check if any errors occured during file i/o
+// Now check if any errors occurred during file i/o
 //
    if ((retc = cFile.LastError()))
       NoGo = eDest->Emsg("Config", retc, "read config file", ConfigFN);
