@@ -137,9 +137,7 @@ int XrdFfsWcache_create(int fd, int flags)
 
     XrdFfsWcacheFbufs[fd].offset = 0;
     XrdFfsWcacheFbufs[fd].len = 0;
-    // "flag & O_RDONLY" is not equivalant to ! (flags & O_RDWR) && ! (flags & O_WRONLY)
-    if ( ! (flags & O_RDWR) &&     
-         ! (flags & O_WRONLY) &&
+    if ( ((flags & O_ACCMODE) == O_RDONLY) &&
          (flags & O_DIRECT) )  // Limit the usage scenario of the read cache 
     {
         XrdFfsWcacheFbufs[fd].buf = (char*)malloc(XrdFfsRcacheBufsize);
