@@ -30,10 +30,10 @@
   
 #include <fcntl.h>
 #include <grp.h>
-#include <string.h>
-#include <time.h>
+#include <cstring>
+#include <ctime>
 #include <pwd.h>
-#include <stdio.h>
+#include <cstdio>
 #include <unistd.h>
 #include <sys/param.h>
 #include <sys/types.h>
@@ -52,6 +52,7 @@
 #include "XrdOuc/XrdOucTList.hh"
 #include "XrdOuc/XrdOucTokenizer.hh"
 #include "XrdSys/XrdSysE2T.hh"
+#include "XrdSys/XrdSysPlatform.hh"
 #include "XrdSys/XrdSysTimer.hh"
 
 using namespace XrdFrc;
