@@ -365,11 +365,6 @@ int compareHash(
 
 }
 
-
-
-
-
-
 // unquote a string and return a new one
 
 char *unquote(char *str) {
@@ -443,6 +438,10 @@ char *quote(const char *str) {
         strcpy(r + j, "%0D");
         j += 3;
         break;
+      case '=':
+        strcpy(r + j, "%3D");
+        j += 3;
+        break;
       default:
         r[j++] = c;
     }
