@@ -116,6 +116,7 @@ int XrdOssSys::Create(const char *tident, const char *path, mode_t access_mode,
     EPNAME("Create")
     const int AMode = S_IRWXU|S_IRWXG|S_IROTH|S_IXOTH; // 775
     char  local_path[MAXPATHLEN+1], *p, pc;
+    local_path[0] = '\0';
     unsigned long long remotefs;
     int isLink = 0, Missing = 1, retc = 0, datfd;
     XrdOssCreateInfo crInfo(local_path, path, access_mode, Opts);
@@ -292,7 +293,7 @@ int XrdOssSys::Alloc_Local(XrdOssCreateInfo &crInfo, XrdOucEnv &env)
 
 // Simply open the file in the local filesystem, creating it if need be.
 //
-   do {datfd = open(crInfo.Path, O_CREAT|O_TRUNC, crInfo.Amode);}
+   do {datfd = open(crInfo.Path, O_RDWR|O_CREAT|O_TRUNC, crInfo.Amode);}
                while(datfd < 0 && errno == EINTR);
    if (datfd < 0) return -errno;
 
