@@ -1069,7 +1069,7 @@ int Cache::Prepare(const char *curl, int oflags, mode_t mode)
    std::string i_name = f_name + Info::s_infoExtension;
 
    // Do not allow write access.
-   if (oflags & (O_WRONLY | O_RDWR | O_APPEND | O_CREAT))
+   if ((oflags & O_ACCMODE) != O_RDONLY)
    {
       TRACE(Warning, "Prepare write access requested on file " << f_name << ". Denying access.");
       return -EROFS;
