@@ -2,6 +2,9 @@
 #include "XrdPfcTrace.hh"
 #include "XrdPfcInfo.hh"
 
+#include "XrdPfcResourceMonitor.hh"
+#include "XrdPfcPurgePin.hh"
+
 #include "XrdOss/XrdOss.hh"
 
 #include "XrdOuc/XrdOucEnv.hh"
@@ -42,6 +45,7 @@ Configuration::Configuration() :
    m_purgeColdFilesAge(-1),
    m_purgeAgeBasedPeriod(10),
    m_accHistorySize(20),
+   m_dirStatsInterval(1500),
    m_dirStatsMaxDepth(-1),
    m_dirStatsStoreDepth(0),
    m_bufferSize(128*1024),
@@ -236,6 +240,59 @@ bool Cache::xdlib(XrdOucStream &Config)
    return true;
 }
 
+/* Function: xplib
+
+   Purpose:  To parse the directive: purgelib <path> [<parms>]
+
+             <path>  the path of the decision library to be used.
+             <parms> optional parameters to be passed.
+
+
+   Output: true upon success or false upon failure.
+ */
+bool Cache::xplib(XrdOucStream &Config)
+{
+   const char*  val;
+
+   std::string libp;
+   if (! (val = Config.GetWord()) || ! val[0])
+   {
+      TRACE(Info," Cache::Config() purgelib not specified; will use LRU for purging files");
+      return true;
+   }
+   else
+   {
+      libp = val;
+   }
+
+   char params[4096];
+   if (val[0])
+      Config.GetRest(params, 4096);
+   else
+      params[0] = 0;
+
+   XrdOucPinLoader* myLib = new XrdOucPinLoader(&m_log, 0, "purgelib",
+                                                libp.c_str());
+
+   PurgePin *(*ep)(XrdSysError&);
+   ep = (PurgePin *(*)(XrdSysError&))myLib->Resolve("XrdPfcGetPurgePin");
+   if (! ep) {myLib->Unload(true); return false; }
+
+   PurgePin * dp = ep(m_log);
+   if (! dp)
+   {
+      TRACE(Error, "Config() purgelib was not able to create a Purge Plugin object?");
+      return false;
+   }
+   m_purge_pin = dp;
+
+   if (params[0])
+      m_purge_pin->ConfigPurgePin(params);
+
+
+   return true;
+}
+
 /* Function: xtrace
 
    Purpose:  To parse the directive: trace <level>
@@ -412,6 +469,10 @@ bool Cache::Config(const char *config_filename, const char *parameters)
       {
          retval = xdlib(Config);
       }
+      else if (! strcmp(var,"pfc.purgelib"))
+      {
+         retval = xplib(Config);
+      }
       else if (! strcmp(var,"pfc.trace"))
       {
          retval = xtrace(Config);
@@ -463,6 +524,18 @@ bool Cache::Config(const char *config_filename, const char *parameters)
    // sets default value for disk usage
    XrdOssVSInfo sP;
    {
+      if (m_configuration.m_meta_space != m_configuration.m_data_space &&
+          m_oss->StatVS(&sP, m_configuration.m_meta_space.c_str(), 1) < 0)
+      {
+         m_log.Emsg("ConfigParameters()", "error obtaining stat info for meta space ", m_configuration.m_meta_space.c_str());
+         return false;
+      }
+      if (m_configuration.m_meta_space != m_configuration.m_data_space && sP.Total < 10ll << 20)
+      {
+         m_log.Emsg("ConfigParameters()", "available data space is less than 10 MB (can be due to a mistake in oss.localroot directive) for space ",
+                    m_configuration.m_meta_space.c_str());
+                    return false;
+      }
       if (m_oss->StatVS(&sP, m_configuration.m_data_space.c_str(), 1) < 0)
       {
          m_log.Emsg("ConfigParameters()", "error obtaining stat info for data space ", m_configuration.m_data_space.c_str());
@@ -500,6 +573,13 @@ bool Cache::Config(const char *config_filename, const char *parameters)
             m_log.Emsg("ConfigParameters()", "pfc.diskusage files should have baseline < nominal < max.");
             aOK = false;
           }
+
+
+         if (aOK && m_configuration.m_fileUsageMax >= m_configuration.m_diskUsageLWM)
+         {
+            m_log.Emsg("ConfigParameters()", "pfc.diskusage files values must be below lowWatermark");
+            aOK = false;
+         }
         }
         else aOK = false;
       }
@@ -595,8 +675,8 @@ bool Cache::Config(const char *config_filename, const char *parameters)
       if (m_configuration.is_dir_stat_reporting_on())
       {
          loff += snprintf(buff + loff, sizeof(buff) - loff,
-                          "       pfc.dirstats maxdepth %d ((internal: store_depth %d, size_of_dirlist %d, size_of_globlist %d))\n",
-                          m_configuration.m_dirStatsMaxDepth, m_configuration.m_dirStatsStoreDepth,
+                          "       pfc.dirstats interval %d maxdepth %d ((internal: store_depth %d, size_of_dirlist %d, size_of_globlist %d))\n",
+                          m_configuration.m_dirStatsInterval, m_configuration.m_dirStatsMaxDepth, m_configuration.m_dirStatsStoreDepth,
                           (int) m_configuration.m_dirStatsDirs.size(), (int) m_configuration.m_dirStatsDirGlobs.size());
          loff += snprintf(buff + loff, sizeof(buff) - loff, "           dirlist:\n");
          for (std::set<std::string>::iterator i = m_configuration.m_dirStatsDirs.begin(); i != m_configuration.m_dirStatsDirs.end(); ++i)
@@ -633,9 +713,16 @@ bool Cache::Config(const char *config_filename, const char *parameters)
 
    m_gstream = (XrdXrootdGStream*) m_env->GetPtr("pfc.gStream*");
 
-   m_log.Say("Config Proxy File Cache g-stream has", m_gstream ? "" : " NOT", " been configured via xrootd.monitor directive");
+   m_log.Say("       pfc g-stream has", m_gstream ? "" : " NOT", " been configured via xrootd.monitor directive\n");
 
-   m_log.Say("------ Proxy File Cache configuration parsing ", aOK ? "completed" : "failed");
+   // Create the ResourceMonitor and get it ready for starting the main thread function.
+   if (aOK)
+   {
+      m_res_mon = new ResourceMonitor(*m_oss);
+      m_res_mon->init_before_main();
+   }
+
+   m_log.Say("=====> Proxy file cache configuration parsing ", aOK ? "completed" : "failed");
 
    if (ofsCfg) delete ofsCfg;
 
@@ -748,7 +835,32 @@ bool Cache::ConfigParameters(std::string part, XrdOucStream& config, TmpConfigur
       const char *p = 0;
       while ((p = cwg.GetWord()) && cwg.HasLast())
       {
-         if (strcmp(p, "maxdepth") == 0)
+         if (strcmp(p, "interval") == 0)
+         {
+            if (XrdOuca2x::a2i(m_log, "Error getting dirstsat interval", cwg.GetWord(), &m_configuration.m_dirStatsInterval, 0, 7 * 24 * 3600))
+            {
+               return false;
+            }
+            int validIntervals[] = {60, 300, 600, 900, 1800, 3600};
+            int size = sizeof(validIntervals) / sizeof(int);
+            bool match = false;
+            std::string vvl;
+            for (int i = 0; i < size; i++) {
+              if (validIntervals[i] == m_configuration.m_dirStatsInterval) {
+                 match = true;
+                 break;
+              }
+              vvl += std::to_string(validIntervals[i]);
+              if ((i+1) != size) vvl += ", ";
+            }
+
+            if (!match) {
+                m_log.Emsg("Config", "Error: Dirstat interval is not valid. Possible interval values are ", vvl.c_str());
+                return false;
+            }
+
+         }
+         else if (strcmp(p, "maxdepth") == 0)
          {
             if (XrdOuca2x::a2i(m_log, "Error getting maxdepth value", cwg.GetWord(), &m_configuration.m_dirStatsMaxDepth, 0, 16))
             {
