@@ -29,8 +29,8 @@
 /******************************************************************************/
 
 #include <fcntl.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdlib>
+#include <cstring>
 #include <unistd.h>
 #include <netinet/in.h>
 #include <sys/types.h>
@@ -50,7 +50,7 @@
 /*                     G l o b a l s   &   S t a t i c s                      */
 /******************************************************************************/
 
-extern XrdOucTrace     *XrdXrootdTrace;
+extern XrdSysTrace      XrdXrootdTrace;
 
        XrdSysError     *XrdXrootdAdmin::eDest;
 
