@@ -30,19 +30,27 @@ XrdCl::Log *TestEnv::sLog       = 0;
 //------------------------------------------------------------------------------
 TestEnv::TestEnv()
 {
-  PutString( "MainServerURL",    "localhost:1094" );
-  PutString( "Manager1URL",      "man1:1094" );
-  PutString( "Manager2URL",      "man2:1094" );
-  PutString( "DiskServerURL",    "localhost:1094" );
+  PutString( "MainServerURL",    "localhost:10940" );
+  PutString( "Manager1URL",      "localhost:10941" );
+  PutString( "Manager2URL",      "localhost:10942" );
+  PutString( "Server1URL",       "localhost:10943" );
+  PutString( "Server2URL",       "localhost:10944" );
+  PutString( "Server3URL",       "localhost:10945" );
+  PutString( "Server4URL",       "localhost:10946" );
+  PutString( "DiskServerURL",    "localhost:10940" );
   PutString( "DataPath",         "/data"         );
   PutString( "RemoteFile",       "/data/cb4aacf1-6f28-42f2-b68a-90a73460f424.dat" );
   PutString( "LocalFile",        "/data/testFile.dat" );
   PutString( "MultiIPServerURL", "multiip:1099" );
-
+  PutString( "LocalDataPath",    "../cluster/data" );
   ImportString( "MainServerURL",    "XRDTEST_MAINSERVERURL" );
   ImportString( "DiskServerURL",    "XRDTEST_DISKSERVERURL" );
   ImportString( "Manager1URL",      "XRDTEST_MANAGER1URL" );
   ImportString( "Manager2URL",      "XRDTEST_MANAGER2URL" );
+  ImportString( "Server1URL",       "XRDTEST_SERVER1URL" );
+  ImportString( "Server2URL",       "XRDTEST_SERVER2URL" );
+  ImportString( "Server3URL",       "XRDTEST_SERVER3URL" );
+  ImportString( "Server4URL",       "XRDTEST_SERVER4URL" );
   ImportString( "DataPath",         "XRDTEST_DATAPATH" );
   ImportString( "LocalFile",        "XRDTEST_LOCALFILE" );
   ImportString( "RemoteFile",       "XRDTEST_REMOTEFILE" );
